@@ -2,19 +2,19 @@ r"""Colored strings that behave mostly like strings
 
 >>> s = fmtstr("Hey there!", 'red')
 >>> s
-red("Hey there!")
+red('Hey there!')
 >>> s[4:7]
-red("the")
+red('the')
 >>> red_on_blue = fmtstr('hello', 'red', 'on_blue')
 >>> blue_on_red = fmtstr('there', fg='blue', bg='red')
 >>> green = fmtstr('!', 'green')
 >>> full = red_on_blue + ' ' + blue_on_red + green
 >>> full
-on_blue(red("hello"))+" "+on_red(blue("there"))+green("!")
+on_blue(red('hello'))+' '+on_red(blue('there'))+green('!')
 >>> str(full)
 '\x1b[31m\x1b[44mhello\x1b[49m\x1b[39m \x1b[34m\x1b[41mthere\x1b[49m\x1b[39m\x1b[32m!\x1b[39m'
 >>> fmtstr(', ').join(['a', fmtstr('b'), fmtstr('c', 'blue')])
-"a"+", "+"b"+", "+blue("c")
+'a'+', '+'b'+', '+blue('c')
 """
 #TODO add a way to composite text without losing original formatting information
 
@@ -41,12 +41,23 @@ xforms = {
     'invert' : lambda x: seq(STYLES['invert'])+x+seq(RESET_ALL),
 }
 
+class FrozenDict(dict):
+    """Immutable dictionary class"""
+    def __setitem__(self, key, value):
+        raise Exception("Cannot change value.")
+
 class BaseFmtStr(object):
     """Formatting annotations on a string"""
     def __init__(self, string, atts=None):
         self._s = string
-        self.atts = dict((k,v) for k,v in atts.items()) if atts else {}
+        self._atts = tuple(atts.items()) if atts else tuple()
 
+    def _get_atts(self):
+        return FrozenDict(self._atts)
+
+    atts = property(_get_atts, None, None, 
+                    'A copy of the current attributes dictionary')
+                     
     s = property(lambda self: self._s) #makes self.s immutable
 
     def __len__(self):
@@ -92,7 +103,7 @@ class BaseFmtStr(object):
         return (''.join(
                         pp_att(att)+'('
                         for att in sorted(self.atts)) +
-               ('"%s"' % self.s) + ')'*len(self.atts))
+               ('%r' % self.s) + ')'*len(self.atts))
 
 class FmtStr(object):
     def __init__(self, *components):
@@ -111,9 +122,9 @@ class FmtStr(object):
     def from_str(cls, s):
         r"""
         >>> fmtstr("|"+fmtstr("hey", fg='red', bg='blue')+"|")
-        "|"+on_blue(red("hey"))+"|"
+        '|'+on_blue(red('hey'))+'|'
         >>> fmtstr('|\x1b[31m\x1b[44mhey\x1b[49m\x1b[39m|')
-        "|"+on_blue(red("hey"))+"|"
+        '|'+on_blue(red('hey'))+'|'
         """
         if '\x1b[' in s:
             tokens_and_strings = parse(s)
@@ -134,9 +145,12 @@ class FmtStr(object):
     def set_attributes(self, **attributes):
         self._unicode = None
         self._str = None
-        for k, v in attributes.items():
-            for fs in self.basefmtstrs:
-                fs.atts[k] = v
+        new_basefmtstrs = []
+        for fs in self.basefmtstrs:
+            new_atts = fs.atts
+            new_atts.update(attributes)
+            new_basefmtstrs.append(BaseFmtStr(fs.s, new_atts)) 
+        self.basefmtstrs = new_basefmtstrs
 
     def join(self, iterable):
         iterable = list(iterable)
@@ -363,7 +377,7 @@ def fmtstr(string, *args, **kwargs):
     Convenience function for creating a FmtStr
 
     >>> fmtstr('asdf', 'blue', 'on_red')
-    on_red(blue("asdf"))
+    on_red(blue('asdf'))
     """
     atts = parse_args(args, kwargs)
     if isinstance(string, FmtStr):
